package designer;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.table.*;

import org.netbeans.lib.awtextra.AbsoluteConstraints;

// import org.netbeans.lib.awtextra.*;

public class DesignerWindow extends javax.swing.JFrame
{
    public DesignerWindow ()
    {
        initComponents ();
        
        addComponents ();
        addColors ();

        DesignerDropListener listener = new DesignerDropListener ();
        DropTarget target = new DropTarget (designer, listener);
        designer.setDropTarget (target);
        
        designer.setLayout (null);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {

        toolbarTabs = new javax.swing.JTabbedPane();
        componentToolbar = new javax.swing.JToolBar();
        colorToolbar = new javax.swing.JToolBar();
        designer = new javax.swing.JPanel();
        mainMenu = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        quitMenuItem = new javax.swing.JMenuItem();
        editMenu = new javax.swing.JMenu();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        componentToolbar.setRollover(true);
        toolbarTabs.addTab("Components", componentToolbar);

        colorToolbar.setRollover(true);
        toolbarTabs.addTab("Colors", colorToolbar);

        designer.setMinimumSize(new java.awt.Dimension(200, 200));
        designer.setPreferredSize(new java.awt.Dimension(200, 276));
        designer.setLayout(null);

        fileMenu.setMnemonic('F');
        fileMenu.setText("File");

        quitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, java.awt.event.InputEvent.CTRL_DOWN_MASK));
        quitMenuItem.setMnemonic('Q');
        quitMenuItem.setText("Quit");
        quitMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                quitClick(evt);
            }
        });
        fileMenu.add(quitMenuItem);

        mainMenu.add(fileMenu);

        editMenu.setMnemonic('E');
        editMenu.setText("Edit");
        mainMenu.add(editMenu);

        setJMenuBar(mainMenu);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(toolbarTabs)
            .addComponent(designer, javax.swing.GroupLayout.DEFAULT_SIZE, 432, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(toolbarTabs, javax.swing.GroupLayout.PREFERRED_SIZE, 63, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(designer, javax.swing.GroupLayout.DEFAULT_SIZE, 200, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /* Tooolbar */
    
    class LocalMouseMotion extends MouseMotionAdapter
    {
        public void mouseDragged (java.awt.event.MouseEvent evt)
        {
            JButton button = (JButton) evt.getSource ();
            TransferHandler handle = button.getTransferHandler ();
            handle.exportAsDrag (button, evt, TransferHandler.COPY);
        }
    }
    
    private void addComponent (String name)
    {
        JButton btn = new JButton ();
        btn.setText (name);
        btn.setTransferHandler (new TransferHandler ("text"));
        btn.addMouseMotionListener (new LocalMouseMotion ());
        componentToolbar.add (btn);
    }
    
    private void addComponents ()
    {
       addComponent ("JButton");
       addComponent ("JTree");
       addComponent ("JTable");
       addComponent ("JTextArea");
    }
    
    private void addColor (Color c)
    {
        JButton btn = new JButton ();
        btn.setText ("   ");
        btn.setBackground (c);
        btn.setTransferHandler (new TransferHandler ("background"));
        
        btn.addMouseMotionListener 
        (new java.awt.event.MouseMotionAdapter () 
         {
            public void mouseDragged(java.awt.event.MouseEvent evt) 
            {
                JButton button = (JButton) evt.getSource();
                TransferHandler handle = button.getTransferHandler();
                handle.exportAsDrag (button, evt, TransferHandler.COPY);
            }
         }
        );
        colorToolbar.add (btn);
    }
    
    private void addColors ()
    {
       addColor (new Color (255, 0, 0));
       addColor (new Color (0, 255, 0))    ;
       addColor (new Color (0, 0, 255))    ;
       addColor (new Color (255, 255, 0))    ;
    }
    
    /* Designer */

    class DesignerDropListener implements DropTargetListener 
    {
        public void drop (DropTargetDropEvent evt)
        {
            // System.out.println ("drop");
            Transferable transfer = evt.getTransferable();
            DataFlavor [] flavors = transfer.getTransferDataFlavors ();
            
            /*
            for (DataFlavor f : flavors)
            {
               System.out.println ("flavor " + f);
               System.out.println ("class " + f.getRepresentationClass ());
            }
            */
            
            if (flavors.length == 1)
            {
                DataFlavor flavor = flavors[0];
                Class cls = flavor.getRepresentationClass ();
                Object data = null;
                try
                {
                    data = transfer.getTransferData (flavor);
                } 
                catch (UnsupportedFlavorException ex) { System.out.println (ex); }
                catch (IOException ex) { System.out.println (ex); }
                    
                if (data != null)
                {
                    Point point = evt.getLocation ();
                    if (cls == String.class)
                    {
                        addDesignerObject (point, data.toString ());
                    }
                    if (cls == Color.class)
                    {
                        Color color = (Color) data;
                        Component comp = designer.getComponentAt (point);
                        comp.setBackground (color);
                    }
                }
            } 
        };

        public void dragEnter(DropTargetDragEvent dtde) { }
        public void dragOver(DropTargetDragEvent dtde) { }
        public void dropActionChanged(DropTargetDragEvent dtde) { }
        public void dragExit(DropTargetEvent dte) { }
    };
    
    private void addDesignerObject (Point point, String toolName)
    {
        Object obj = null;
        try 
        {
            toolName = "javax.swing." + toolName;
            Class cls = Class.forName (toolName);
            obj = cls.newInstance();
            // System.out.println ("new object " + obj);
        } 
        catch (ClassNotFoundException ex) { System.out.println (ex); } 
        catch (InstantiationException ex) { System.out.println (ex); }
        catch (IllegalAccessException ex) { System.out.println (ex); }
        
        if (obj instanceof Component)
        {
            Component comp = (Component) obj;
            designer.add (comp);
            comp.setBounds (point.x, point.y, 100, 100);
            
            if (comp instanceof JButton)
            {
               ((JButton)comp).setText ("text");
               // System.out.println ("text");
            }
            if (comp instanceof JTree)
            {
               // ((JTree)comp).setMinimumSize(new Dimension (100, 100));
               // System.out.println ("tree");
            }
            if (comp instanceof JTable)
            {
               JTable table = (JTable) comp;
                DefaultTableModel m = (DefaultTableModel) table.getModel ();
                m.setColumnCount (2);
                m.addRow (new String [] {"1", "abc" });
                m.addRow (new String [] {"2", "klm" });
               // System.out.println ("table");
            }
            if (comp instanceof JTextArea)
            {
               ((JTextArea)comp).setText("some text");
               // System.out.println ("edit");
            }
        }
    }
   
    
    private void quitClick(java.awt.event.ActionEvent evt)//GEN-FIRST:event_quitClick
    {//GEN-HEADEREND:event_quitClick
        System.exit (0);
    }//GEN-LAST:event_quitClick

    public static void main (String args[])
    {
        /* Do not use Nimbus look and feel */

        /* Create and display the form */
        java.awt.EventQueue.invokeLater (new Runnable ()
        {
            public void run ()
            {
                new DesignerWindow ().setVisible (true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JToolBar colorToolbar;
    private javax.swing.JToolBar componentToolbar;
    private javax.swing.JPanel designer;
    private javax.swing.JMenu editMenu;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenuBar mainMenu;
    private javax.swing.JMenuItem quitMenuItem;
    private javax.swing.JTabbedPane toolbarTabs;
    // End of variables declaration//GEN-END:variables
}
